# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""Provides interfaces to various longitudinal commands provided by freesurfer
"""
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)

import os
import os.path

from ... import logging
from ...utils.filemanip import split_filename, copyfile

from .base import (FSCommand, FSTraitedSpec, FSScriptCommand,
                   FSScriptOutputSpec, FSCommandOpenMP, FSTraitedSpecOpenMP)
from ..base import (isdefined, TraitedSpec, File, traits, Directory)

__docformat__ = 'restructuredtext'
iflogger = logging.getLogger('nipype.interface')


class MPRtoMNI305InputSpec(FSTraitedSpec):
    # environment variables, required
    # usedefault=True is hack for on_trait_change in __init__
    reference_dir = Directory(
        "", exists=True, mandatory=True, usedefault=True, desc="TODO")
    target = traits.String(
        "", mandatory=True, usedefault=True, desc="input atlas file")
    # required
    in_file = File(
        argstr='%s',
        usedefault=True,
        desc="the input file prefix for MPRtoMNI305")


class MPRtoMNI305OutputSpec(FSScriptOutputSpec):
    out_file = File(
        exists=False,
        desc="The output file '<in_file>_to_<target>_t4_vox2vox.txt'")


class MPRtoMNI305(FSScriptCommand):
    """
    For complete details, see FreeSurfer documentation

    Examples
    ========

    >>> from nipype.interfaces.freesurfer import MPRtoMNI305, Info
    >>> mprtomni305 = MPRtoMNI305()
    >>> mprtomni305.inputs.target = 'structural.nii'
    >>> mprtomni305.inputs.reference_dir = '.' # doctest: +SKIP
    >>> mprtomni305.cmdline # doctest: +SKIP
    'mpr2mni305 output'
    >>> mprtomni305.inputs.out_file = 'struct_out' # doctest: +SKIP
    >>> mprtomni305.cmdline # doctest: +SKIP
    'mpr2mni305 struct_out' # doctest: +SKIP
    >>> mprtomni305.inputs.environ['REFDIR'] == os.path.join(Info.home(), 'average') # doctest: +SKIP
    True
    >>> mprtomni305.inputs.environ['MPR2MNI305_TARGET'] # doctest: +SKIP
    'structural'
    >>> mprtomni305.run() # doctest: +SKIP

    """
    _cmd = "mpr2mni305"
    input_spec = MPRtoMNI305InputSpec
    output_spec = MPRtoMNI305OutputSpec

    def __init__(self, **inputs):
        super(MPRtoMNI305, self).__init__(**inputs)
        self.inputs.on_trait_change(self._environ_update, 'target')
        self.inputs.on_trait_change(self._environ_update, 'reference_dir')

    def _format_arg(self, opt, spec, val):
        if opt in ['target', 'reference_dir']:
            return ""
        elif opt == 'in_file':
            _, retval, ext = split_filename(val)
            # Need to copy file to working cache directory!
            copyfile(
                val,
                os.path.abspath(retval + ext),
                copy=True,
                hashmethod='content')
            return retval
        return super(MPRtoMNI305, self)._format_arg(opt, spec, val)

    def _environ_update(self):
        # refdir = os.path.join(Info.home(), val)
        refdir = self.inputs.reference_dir
        target = self.inputs.target
        self.inputs.environ['MPR2MNI305_TARGET'] = target
        self.inputs.environ["REFDIR"] = refdir

    def _get_fname(self, fname):
        return split_filename(fname)[1]

    def _list_outputs(self):
        outputs = super(MPRtoMNI305, self)._list_outputs()
        fullname = "_".join([
            self._get_fname(self.inputs.in_file), "to", self.inputs.target,
            "t4", "vox2vox.txt"
        ])
        outputs['out_file'] = os.path.abspath(fullname)
        return outputs


class RegisterAVItoTalairachInputSpec(FSTraitedSpec):
    in_file = File(
        argstr='%s',
        exists=True,
        mandatory=True,
        position=0,
        desc="The input file")
    target = File(
        argstr='%s',
        exists=True,
        mandatory=True,
        position=1,
        desc="The target file")
    vox2vox = File(
        argstr='%s',
        exists=True,
        mandatory=True,
        position=2,
        desc="The vox2vox file")
    out_file = File(
        'talairach.auto.xfm',
        usedefault=True,
        argstr='%s',
        position=3,
        desc="The transform output")


class RegisterAVItoTalairachOutputSpec(FSScriptOutputSpec):
    out_file = File(
        exists=False, desc="The output file for RegisterAVItoTalairach")


class RegisterAVItoTalairach(FSScriptCommand):
    """
    converts the vox2vox from talairach_avi to a talairach.xfm file

    This is a script that converts the vox2vox from talairach_avi to a
    talairach.xfm file. It is meant to replace the following cmd line:

    tkregister2_cmdl \
        --mov $InVol \
        --targ $FREESURFER_HOME/average/mni305.cor.mgz \
        --xfmout ${XFM} \
        --vox2vox talsrcimg_to_${target}_t4_vox2vox.txt \
        --noedit \
        --reg talsrcimg.reg.tmp.dat
    set targ = $FREESURFER_HOME/average/mni305.cor.mgz
    set subject = mgh-02407836-v2
    set InVol = $SUBJECTS_DIR/$subject/mri/orig.mgz
    set vox2vox = $SUBJECTS_DIR/$subject/mri/transforms/talsrcimg_to_711-2C_as_mni_average_305_t4_vox2vox.txt

    Examples
    ========

    >>> from nipype.interfaces.freesurfer import RegisterAVItoTalairach
    >>> register = RegisterAVItoTalairach()
    >>> register.inputs.in_file = 'structural.mgz'                         # doctest: +SKIP
    >>> register.inputs.target = 'mni305.cor.mgz'                          # doctest: +SKIP
    >>> register.inputs.vox2vox = 'talsrcimg_to_structural_t4_vox2vox.txt' # doctest: +SKIP
    >>> register.cmdline                                                   # doctest: +SKIP
    'avi2talxfm structural.mgz mni305.cor.mgz talsrcimg_to_structural_t4_vox2vox.txt talairach.auto.xfm'

    >>> register.run() # doctest: +SKIP
    """
    _cmd = "avi2talxfm"
    input_spec = RegisterAVItoTalairachInputSpec
    output_spec = RegisterAVItoTalairachOutputSpec

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outputs['out_file'] = os.path.abspath(self.inputs.out_file)
        return outputs


class EMRegisterInputSpec(FSTraitedSpecOpenMP):
    # required
    in_file = File(
        argstr="%s",
        exists=True,
        mandatory=True,
        position=-3,
        desc="in brain volume")
    template = File(
        argstr="%s",
        exists=True,
        mandatory=True,
        position=-2,
        desc="template gca")
    out_file = File(
        argstr="%s",
        exists=False,
        name_source=['in_file'],
        name_template="%s_transform.lta",
        hash_files=False,
        keep_extension=False,
        position=-1,
        desc="output transform")
    # optional
    skull = traits.Bool(
        argstr="-skull", desc="align to atlas containing skull (uns=5)")
    mask = File(argstr="-mask %s", exists=True, desc="use volume as a mask")
    nbrspacing = traits.Int(
        argstr="-uns %d",
        desc=
        "align to atlas containing skull setting unknown_nbr_spacing = nbrspacing"
    )
    transform = File(
        argstr="-t %s", exists=True, desc="Previously computed transform")


class EMRegisterOutputSpec(TraitedSpec):
    out_file = File(exists=False, desc="output transform")


class EMRegister(FSCommandOpenMP):
    """ This program creates a tranform in lta format

    Examples
    ========
    >>> from nipype.interfaces.freesurfer import EMRegister
    >>> register = EMRegister()
    >>> register.inputs.in_file = 'norm.mgz'
    >>> register.inputs.template = 'aseg.mgz'
    >>> register.inputs.out_file = 'norm_transform.lta'
    >>> register.inputs.skull = True
    >>> register.inputs.nbrspacing = 9
    >>> register.cmdline
    'mri_em_register -uns 9 -skull norm.mgz aseg.mgz norm_transform.lta'
    """
    _cmd = 'mri_em_register'
    input_spec = EMRegisterInputSpec
    output_spec = EMRegisterOutputSpec

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outputs['out_file'] = os.path.abspath(self.inputs.out_file)
        return outputs


class RegisterInputSpec(FSTraitedSpec):
    # required
    in_surf = File(
        argstr="%s",
        exists=True,
        mandatory=True,
        position=-3,
        copyfile=True,
        desc="Surface to register, often {hemi}.sphere")
    target = File(
        argstr="%s",
        exists=True,
        mandatory=True,
        position=-2,
        desc="The data to register to. In normal recon-all usage, " +
        "this is a template file for average surface.")
    in_sulc = File(
        exists=True,
        mandatory=True,
        copyfile=True,
        desc=
        "Undocumented mandatory input file ${SUBJECTS_DIR}/surf/{hemisphere}.sulc "
    )
    out_file = File(
        argstr="%s",
        exists=False,
        position=-1,
        genfile=True,
        desc="Output surface file to capture registration")
    # optional
    curv = traits.Bool(
        argstr="-curv",
        requires=['in_smoothwm'],
        desc="Use smoothwm curvature for final alignment")
    in_smoothwm = File(
        exists=True,
        copyfile=True,
        desc=
        "Undocumented input file ${SUBJECTS_DIR}/surf/{hemisphere}.smoothwm ")


class RegisterOutputSpec(TraitedSpec):
    out_file = File(
        exists=False, desc="Output surface file to capture registration")


class Register(FSCommand):
    """ This program registers a surface to an average surface template.

    Examples
    ========
    >>> from nipype.interfaces.freesurfer import Register
    >>> register = Register()
    >>> register.inputs.in_surf = 'lh.pial'
    >>> register.inputs.in_smoothwm = 'lh.pial'
    >>> register.inputs.in_sulc = 'lh.pial'
    >>> register.inputs.target = 'aseg.mgz'
    >>> register.inputs.out_file = 'lh.pial.reg'
    >>> register.inputs.curv = True
    >>> register.cmdline
    'mris_register -curv lh.pial aseg.mgz lh.pial.reg'
    """

    _cmd = 'mris_register'
    input_spec = RegisterInputSpec
    output_spec = RegisterOutputSpec

    def _format_arg(self, opt, spec, val):
        if opt == 'curv':
            return spec.argstr
        return super(Register, self)._format_arg(opt, spec, val)

    def _gen_filename(self, name):
        if name == 'out_file':
            return self._list_outputs()[name]
        return None

    def _list_outputs(self):
        outputs = self.output_spec().get()
        if isdefined(self.inputs.out_file):
            outputs['out_file'] = os.path.abspath(self.inputs.out_file)
        else:
            outputs['out_file'] = os.path.abspath(self.inputs.in_surf) + '.reg'
        return outputs


class PaintInputSpec(FSTraitedSpec):
    # required
    in_surf = File(
        argstr="%s",
        exists=True,
        mandatory=True,
        position=-2,
        desc="Surface file with grid (vertices) onto which the " +
        "template data is to be sampled or 'painted'")
    template = File(
        argstr="%s",
        exists=True,
        mandatory=True,
        position=-3,
        desc="Template file")
    # optional
    template_param = traits.Int(desc="Frame number of the input template")
    averages = traits.Int(argstr="-a %d", desc="Average curvature patterns")
    out_file = File(
        argstr="%s",
        exists=False,
        position=-1,
        name_template="%s.avg_curv",
        hash_files=False,
        name_source=['in_surf'],
        keep_extension=False,
        desc="File containing a surface-worth of per-vertex values, " +
        "saved in 'curvature' format.")


class PaintOutputSpec(TraitedSpec):
    out_file = File(
        exists=False,
        desc=
        "File containing a surface-worth of per-vertex values, saved in 'curvature' format."
    )


class Paint(FSCommand):
    """
    This program is useful for extracting one of the arrays ("a variable")
    from a surface-registration template file. The output is a file
    containing a surface-worth of per-vertex values, saved in "curvature"
    format. Because the template data is sampled to a particular surface
    mesh, this conjures the idea of "painting to a surface".

    Examples
    ========
    >>> from nipype.interfaces.freesurfer import Paint
    >>> paint = Paint()
    >>> paint.inputs.in_surf = 'lh.pial'
    >>> paint.inputs.template = 'aseg.mgz'
    >>> paint.inputs.averages = 5
    >>> paint.inputs.out_file = 'lh.avg_curv'
    >>> paint.cmdline
    'mrisp_paint -a 5 aseg.mgz lh.pial lh.avg_curv'
    """

    _cmd = 'mrisp_paint'
    input_spec = PaintInputSpec
    output_spec = PaintOutputSpec

    def _format_arg(self, opt, spec, val):
        if opt == 'template':
            if isdefined(self.inputs.template_param):
                return spec.argstr % (
                    val + '#' + str(self.inputs.template_param))
        return super(Paint, self)._format_arg(opt, spec, val)

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outputs['out_file'] = os.path.abspath(self.inputs.out_file)
        return outputs


class MRICoregInputSpec(FSTraitedSpec):
    source_file = File(
        argstr='--mov %s',
        desc='source file to be registered',
        mandatory=True,
        copyfile=False)
    reference_file = File(
        argstr='--ref %s',
        desc='reference (target) file',
        mandatory=True,
        copyfile=False,
        xor=['subject_id'])
    out_lta_file = traits.Either(
        True,
        File,
        argstr='--lta %s',
        default=True,
        usedefault=True,
        desc='output registration file (LTA format)')
    out_reg_file = traits.Either(
        True,
        File,
        argstr='--regdat %s',
        desc='output registration file (REG format)')
    out_params_file = traits.Either(
        True, File, argstr='--params %s', desc='output parameters file')

    subjects_dir = Directory(
        exists=True, argstr='--sd %s', desc='FreeSurfer SUBJECTS_DIR')
    subject_id = traits.Str(
        argstr='--s %s',
        position=1,
        mandatory=True,
        xor=['reference_file'],
        requires=['subjects_dir'],
        desc='freesurfer subject ID (implies ``reference_mask == '
        'aparc+aseg.mgz`` unless otherwise specified)')
    dof = traits.Enum(
        6,
        9,
        12,
        argstr='--dof %d',
        desc='number of transform degrees of freedom')
    reference_mask = traits.Either(
        False,
        traits.Str,
        argstr='--ref-mask %s',
        position=2,
        desc='mask reference volume with given mask, or None if ``False``')
    source_mask = traits.Str(
        argstr='--mov-mask', desc='mask source file with given mask')
    num_threads = traits.Int(
        argstr='--threads %d', desc='number of OpenMP threads')
    no_coord_dithering = traits.Bool(
        argstr='--no-coord-dither', desc='turn off coordinate dithering')
    no_intensity_dithering = traits.Bool(
        argstr='--no-intensity-dither', desc='turn off intensity dithering')
    sep = traits.List(
        argstr='--sep %s...',
        minlen=1,
        maxlen=2,
        desc='set spatial scales, in voxels (default [2, 4])')
    initial_translation = traits.Tuple(
        traits.Float,
        traits.Float,
        traits.Float,
        argstr='--trans %g %g %g',
        desc='initial translation in mm (implies no_cras0)')
    initial_rotation = traits.Tuple(
        traits.Float,
        traits.Float,
        traits.Float,
        argstr='--rot %g %g %g',
        desc='initial rotation in degrees')
    initial_scale = traits.Tuple(
        traits.Float,
        traits.Float,
        traits.Float,
        argstr='--scale %g %g %g',
        desc='initial scale')
    initial_shear = traits.Tuple(
        traits.Float,
        traits.Float,
        traits.Float,
        argstr='--shear %g %g %g',
        desc='initial shear (Hxy, Hxz, Hyz)')
    no_cras0 = traits.Bool(
        argstr='--no-cras0',
        desc='do not set translation parameters to align '
        'centers of source and reference files')
    max_iters = traits.Range(
        low=1, argstr='--nitersmax %d', desc='maximum iterations (default: 4)')
    ftol = traits.Float(
        argstr='--ftol %e', desc='floating-point tolerance (default=1e-7)')
    linmintol = traits.Float(argstr='--linmintol %e')
    saturation_threshold = traits.Range(
        low=0.0,
        high=100.0,
        argstr='--sat %g',
        desc='saturation threshold (default=9.999)')
    conform_reference = traits.Bool(
        argstr='--conf-ref', desc='conform reference without rescaling')
    no_brute_force = traits.Bool(
        argstr='--no-bf', desc='do not brute force search')
    brute_force_limit = traits.Float(
        argstr='--bf-lim %g',
        xor=['no_brute_force'],
        desc='constrain brute force search to +/- lim')
    brute_force_samples = traits.Int(
        argstr='--bf-nsamp %d',
        xor=['no_brute_force'],
        desc='number of samples in brute force search')
    no_smooth = traits.Bool(
        argstr='--no-smooth',
        desc='do not apply smoothing to either reference or source file')
    ref_fwhm = traits.Float(
        argstr='--ref-fwhm', desc='apply smoothing to reference file')
    source_oob = traits.Bool(
        argstr='--mov-oob',
        desc='count source voxels that are out-of-bounds as 0')
    # Skipping mat2par


class MRICoregOutputSpec(TraitedSpec):
    out_reg_file = File(exists=True, desc='output registration file')
    out_lta_file = File(exists=True, desc='output LTA-style registration file')
    out_params_file = File(exists=True, desc='output parameters file')


class MRICoreg(FSCommand):
    """ This program registers one volume to another

    mri_coreg is a C reimplementation of spm_coreg in FreeSurfer

    Examples
    ========
    >>> from nipype.interfaces.freesurfer import MRICoreg
    >>> coreg = MRICoreg()
    >>> coreg.inputs.source_file = 'moving1.nii'
    >>> coreg.inputs.reference_file = 'fixed1.nii'
    >>> coreg.inputs.subjects_dir = '.'
    >>> coreg.cmdline # doctest: +ELLIPSIS
    'mri_coreg --lta .../registration.lta --ref fixed1.nii --mov moving1.nii --sd .'

    If passing a subject ID, the reference mask may be disabled:

    >>> coreg = MRICoreg()
    >>> coreg.inputs.source_file = 'moving1.nii'
    >>> coreg.inputs.subjects_dir = '.'
    >>> coreg.inputs.subject_id = 'fsaverage'
    >>> coreg.inputs.reference_mask = False
    >>> coreg.cmdline # doctest: +ELLIPSIS
    'mri_coreg --s fsaverage --no-ref-mask --lta .../registration.lta --mov moving1.nii --sd .'

    Spatial scales may be specified as a list of one or two separations:

    >>> coreg.inputs.sep = [4]
    >>> coreg.cmdline # doctest: +ELLIPSIS
    'mri_coreg --s fsaverage --no-ref-mask --lta .../registration.lta --sep 4 --mov moving1.nii --sd .'

    >>> coreg.inputs.sep = [4, 5]
    >>> coreg.cmdline # doctest: +ELLIPSIS
    'mri_coreg --s fsaverage --no-ref-mask --lta .../registration.lta --sep 4 --sep 5 --mov moving1.nii --sd .'
    """

    _cmd = 'mri_coreg'
    input_spec = MRICoregInputSpec
    output_spec = MRICoregOutputSpec

    def _format_arg(self, opt, spec, val):
        if opt in ('out_reg_file', 'out_lta_file',
                   'out_params_file') and val is True:
            val = self._list_outputs()[opt]
        elif opt == 'reference_mask' and val is False:
            return '--no-ref-mask'
        return super(MRICoreg, self)._format_arg(opt, spec, val)

    def _list_outputs(self):
        outputs = self.output_spec().get()

        out_lta_file = self.inputs.out_lta_file
        if isdefined(out_lta_file):
            if out_lta_file is True:
                out_lta_file = 'registration.lta'
            outputs['out_lta_file'] = os.path.abspath(out_lta_file)

        out_reg_file = self.inputs.out_reg_file
        if isdefined(out_reg_file):
            if out_reg_file is True:
                out_reg_file = 'registration.dat'
            outputs['out_reg_file'] = os.path.abspath(out_reg_file)

        out_params_file = self.inputs.out_params_file
        if isdefined(out_params_file):
            if out_params_file is True:
                out_params_file = 'registration.par'
            outputs['out_params_file'] = os.path.abspath(out_params_file)

        return outputs



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..base import AssertEqual


def test_AssertEqual_inputs():
    input_map = dict(
        volume1=dict(
            extensions=None,
            mandatory=True,
        ),
        volume2=dict(
            extensions=None,
            mandatory=True,
        ),
    )
    inputs = AssertEqual.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..dti import DTIFit


def test_DTIFit_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        base_name=dict(
            argstr='-o %s',
            position=1,
            usedefault=True,
        ),
        bvals=dict(
            argstr='-b %s',
            extensions=None,
            mandatory=True,
            position=4,
        ),
        bvecs=dict(
            argstr='-r %s',
            extensions=None,
            mandatory=True,
            position=3,
        ),
        cni=dict(
            argstr='--cni=%s',
            extensions=None,
        ),
        dwi=dict(
            argstr='-k %s',
            extensions=None,
            mandatory=True,
            position=0,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        gradnonlin=dict(
            argstr='--gradnonlin=%s',
            extensions=None,
        ),
        little_bit=dict(argstr='--littlebit', ),
        mask=dict(
            argstr='-m %s',
            extensions=None,
            mandatory=True,
            position=2,
        ),
        max_x=dict(argstr='-X %d', ),
        max_y=dict(argstr='-Y %d', ),
        max_z=dict(argstr='-Z %d', ),
        min_x=dict(argstr='-x %d', ),
        min_y=dict(argstr='-y %d', ),
        min_z=dict(argstr='-z %d', ),
        output_type=dict(),
        save_tensor=dict(argstr='--save_tensor', ),
        sse=dict(argstr='--sse', ),
    )
    inputs = DTIFit.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_DTIFit_outputs():
    output_map = dict(
        FA=dict(extensions=None, ),
        L1=dict(extensions=None, ),
        L2=dict(extensions=None, ),
        L3=dict(extensions=None, ),
        MD=dict(extensions=None, ),
        MO=dict(extensions=None, ),
        S0=dict(extensions=None, ),
        V1=dict(extensions=None, ),
        V2=dict(extensions=None, ),
        V3=dict(extensions=None, ),
        sse=dict(extensions=None, ),
        tensor=dict(extensions=None, ),
    )
    outputs = DTIFit.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..preprocess import Segment


def test_Segment_inputs():
    input_map = dict(
        affine_regularization=dict(field='opts.regtype', ),
        bias_fwhm=dict(field='opts.biasfwhm', ),
        bias_regularization=dict(field='opts.biasreg', ),
        clean_masks=dict(field='output.cleanup', ),
        csf_output_type=dict(field='output.CSF', ),
        data=dict(
            copyfile=False,
            field='data',
            mandatory=True,
        ),
        gaussians_per_class=dict(field='opts.ngaus', ),
        gm_output_type=dict(field='output.GM', ),
        mask_image=dict(
            extensions=None,
            field='opts.msk',
        ),
        matlab_cmd=dict(),
        mfile=dict(usedefault=True, ),
        paths=dict(),
        sampling_distance=dict(field='opts.samp', ),
        save_bias_corrected=dict(field='output.biascor', ),
        tissue_prob_maps=dict(field='opts.tpm', ),
        use_mcr=dict(),
        use_v8struct=dict(
            min_ver='8',
            usedefault=True,
        ),
        warp_frequency_cutoff=dict(field='opts.warpco', ),
        warping_regularization=dict(field='opts.warpreg', ),
        wm_output_type=dict(field='output.WM', ),
    )
    inputs = Segment.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Segment_outputs():
    output_map = dict(
        bias_corrected_image=dict(extensions=None, ),
        inverse_transformation_mat=dict(extensions=None, ),
        modulated_csf_image=dict(extensions=None, ),
        modulated_gm_image=dict(extensions=None, ),
        modulated_input_image=dict(
            deprecated='0.10',
            extensions=None,
            new_name='bias_corrected_image',
        ),
        modulated_wm_image=dict(extensions=None, ),
        native_csf_image=dict(extensions=None, ),
        native_gm_image=dict(extensions=None, ),
        native_wm_image=dict(extensions=None, ),
        normalized_csf_image=dict(extensions=None, ),
        normalized_gm_image=dict(extensions=None, ),
        normalized_wm_image=dict(extensions=None, ),
        transformation_mat=dict(extensions=None, ),
    )
    outputs = Segment.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..base import DipyBaseInterface


def test_DipyBaseInterface_inputs():
    input_map = dict()
    inputs = DipyBaseInterface.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..registration import MeasureImageSimilarity


def test_MeasureImageSimilarity_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        dimension=dict(
            argstr='--dimensionality %d',
            position=1,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fixed_image=dict(
            extensions=None,
            mandatory=True,
        ),
        fixed_image_mask=dict(
            argstr='%s',
            extensions=None,
        ),
        metric=dict(
            argstr='%s',
            mandatory=True,
        ),
        metric_weight=dict(
            requires=['metric'],
            usedefault=True,
        ),
        moving_image=dict(
            extensions=None,
            mandatory=True,
        ),
        moving_image_mask=dict(
            extensions=None,
            requires=['fixed_image_mask'],
        ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        radius_or_number_of_bins=dict(
            mandatory=True,
            requires=['metric'],
        ),
        sampling_percentage=dict(
            mandatory=True,
            requires=['metric'],
        ),
        sampling_strategy=dict(
            requires=['metric'],
            usedefault=True,
        ),
    )
    inputs = MeasureImageSimilarity.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_MeasureImageSimilarity_outputs():
    output_map = dict(similarity=dict(), )
    outputs = MeasureImageSimilarity.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..tracks import TrackDensityMap


def test_TrackDensityMap_inputs():
    input_map = dict(
        data_dims=dict(),
        in_file=dict(
            extensions=None,
            mandatory=True,
        ),
        out_filename=dict(
            extensions=None,
            usedefault=True,
        ),
        points_space=dict(usedefault=True, ),
        reference=dict(extensions=None, ),
        voxel_dims=dict(),
    )
    inputs = TrackDensityMap.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_TrackDensityMap_outputs():
    output_map = dict(out_file=dict(extensions=None, ), )
    outputs = TrackDensityMap.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..minc import Beast


def test_Beast_inputs():
    input_map = dict(
        abspath=dict(
            argstr='-abspath',
            usedefault=True,
        ),
        args=dict(argstr='%s', ),
        clobber=dict(
            argstr='-clobber',
            usedefault=True,
        ),
        confidence_level_alpha=dict(
            argstr='-alpha %s',
            usedefault=True,
        ),
        configuration_file=dict(
            argstr='-configuration %s',
            extensions=None,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fill_holes=dict(argstr='-fill', ),
        flip_images=dict(argstr='-flip', ),
        input_file=dict(
            argstr='%s',
            extensions=None,
            mandatory=True,
            position=-2,
        ),
        library_dir=dict(
            argstr='%s',
            mandatory=True,
            position=-3,
        ),
        load_moments=dict(argstr='-load_moments', ),
        median_filter=dict(argstr='-median', ),
        nlm_filter=dict(argstr='-nlm_filter', ),
        number_selected_images=dict(
            argstr='-selection_num %s',
            usedefault=True,
        ),
        output_file=dict(
            argstr='%s',
            extensions=None,
            hash_files=False,
            name_source=['input_file'],
            name_template='%s_beast_mask.mnc',
            position=-1,
        ),
        patch_size=dict(
            argstr='-patch_size %s',
            usedefault=True,
        ),
        probability_map=dict(argstr='-probability', ),
        same_resolution=dict(argstr='-same_resolution', ),
        search_area=dict(
            argstr='-search_area %s',
            usedefault=True,
        ),
        smoothness_factor_beta=dict(
            argstr='-beta %s',
            usedefault=True,
        ),
        threshold_patch_selection=dict(
            argstr='-threshold %s',
            usedefault=True,
        ),
        voxel_size=dict(
            argstr='-voxel_size %s',
            usedefault=True,
        ),
    )
    inputs = Beast.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Beast_outputs():
    output_map = dict(output_file=dict(extensions=None, ), )
    outputs = Beast.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)

import os
import os.path as op

from ...utils.filemanip import split_filename
from ..base import (CommandLineInputSpec, CommandLine, traits, TraitedSpec,
                    File, isdefined)


class FilterTracksInputSpec(CommandLineInputSpec):
    in_file = File(
        exists=True,
        argstr='%s',
        mandatory=True,
        position=-2,
        desc='input tracks to be filtered')
    include_xor = ['include_file', 'include_spec']
    include_file = File(
        exists=True,
        argstr='-include %s',
        desc='inclusion file',
        xor=include_xor)
    include_spec = traits.List(
        traits.Float,
        desc='inclusion specification in mm and radius (x y z r)',
        position=2,
        argstr='-include %s',
        minlen=4,
        maxlen=4,
        sep=',',
        units='mm',
        xor=include_xor)

    exclude_xor = ['exclude_file', 'exclude_spec']
    exclude_file = File(
        exists=True,
        argstr='-exclude %s',
        desc='exclusion file',
        xor=exclude_xor)
    exclude_spec = traits.List(
        traits.Float,
        desc='exclusion specification in mm and radius (x y z r)',
        position=2,
        argstr='-exclude %s',
        minlen=4,
        maxlen=4,
        sep=',',
        units='mm',
        xor=exclude_xor)

    minimum_tract_length = traits.Float(
        argstr='-minlength %s',
        units='mm',
        desc=
        "Sets the minimum length of any track in millimeters (default is 10 mm)."
    )

    out_file = File(
        argstr='%s',
        position=-1,
        desc='Output filtered track filename',
        name_source=['in_file'],
        hash_files=False,
        name_template='%s_filt')

    no_mask_interpolation = traits.Bool(
        argstr='-nomaskinterp',
        desc="Turns off trilinear interpolation of mask images.")
    invert = traits.Bool(
        argstr='-invert',
        desc="invert the matching process, so that tracks that would"
        "otherwise have been included are now excluded and vice-versa.")

    quiet = traits.Bool(
        argstr='-quiet',
        position=1,
        desc="Do not display information messages or progress status.")
    debug = traits.Bool(
        argstr='-debug', position=1, desc="Display debugging messages.")


class FilterTracksOutputSpec(TraitedSpec):
    out_file = File(exists=True, desc='the output filtered tracks')


class FilterTracks(CommandLine):
    """
    Use regions-of-interest to select a subset of tracks
    from a given MRtrix track file.

    Example
    -------

    >>> import nipype.interfaces.mrtrix as mrt
    >>> filt = mrt.FilterTracks()
    >>> filt.inputs.in_file = 'tracks.tck'
    >>> filt.run()                                 # doctest: +SKIP
    """

    _cmd = 'filter_tracks'
    input_spec = FilterTracksInputSpec
    output_spec = FilterTracksOutputSpec


class Tracks2ProbInputSpec(CommandLineInputSpec):
    in_file = File(
        exists=True,
        argstr='%s',
        mandatory=True,
        position=-2,
        desc='tract file')
    template_file = File(
        exists=True,
        argstr='-template %s',
        position=1,
        desc=
        'an image file to be used as a template for the output (the output image wil have the same transform and field of view)'
    )
    voxel_dims = traits.List(
        traits.Float,
        argstr='-vox %s',
        sep=',',
        position=2,
        minlen=3,
        maxlen=3,
        desc=
        'Three comma-separated numbers giving the size of each voxel in mm.')
    colour = traits.Bool(
        argstr='-colour',
        position=3,
        desc=
        "add colour to the output image according to the direction of the tracks."
    )
    fraction = traits.Bool(
        argstr='-fraction',
        position=3,
        desc=
        "produce an image of the fraction of fibres through each voxel (as a proportion of the total number in the file), rather than the count."
    )
    output_datatype = traits.Enum(
        "Bit",
        "Int8",
        "UInt8",
        "Int16",
        "UInt16",
        "Int32",
        "UInt32",
        "float32",
        "float64",
        argstr='-datatype %s',
        position=2,
        desc=
        '"i.e. Bfloat". Can be "char", "short", "int", "long", "float" or "double"'
    )  # , usedefault=True)
    resample = traits.Float(
        argstr='-resample %d',
        position=3,
        units='mm',
        desc=
        'resample the tracks at regular intervals using Hermite interpolation. If omitted, the program will select an appropriate interpolation factor automatically.'
    )
    out_filename = File(
        genfile=True, argstr='%s', position=-1, desc='output data file')


class Tracks2ProbOutputSpec(TraitedSpec):
    tract_image = File(
        exists=True, desc='Output tract count or track density image')


class Tracks2Prob(CommandLine):
    """
    Convert a tract file into a map of the fraction of tracks to enter
    each voxel - also known as a tract density image (TDI) - in MRtrix's
    image format (.mif). This can be viewed using MRview or converted to
    Nifti using MRconvert.

    Example
    -------

    >>> import nipype.interfaces.mrtrix as mrt
    >>> tdi = mrt.Tracks2Prob()
    >>> tdi.inputs.in_file = 'dwi_CSD_tracked.tck'
    >>> tdi.inputs.colour = True
    >>> tdi.run()                                       # doctest: +SKIP
    """

    _cmd = 'tracks2prob'
    input_spec = Tracks2ProbInputSpec
    output_spec = Tracks2ProbOutputSpec

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outputs['tract_image'] = self.inputs.out_filename
        if not isdefined(outputs['tract_image']):
            outputs['tract_image'] = op.abspath(self._gen_outfilename())
        else:
            outputs['tract_image'] = os.path.abspath(outputs['tract_image'])
        return outputs

    def _gen_filename(self, name):
        if name == 'out_filename':
            return self._gen_outfilename()
        else:
            return None

    def _gen_outfilename(self):
        _, name, _ = split_filename(self.inputs.in_file)
        return name + '_TDI.mif'


class StreamlineTrackInputSpec(CommandLineInputSpec):
    in_file = File(
        exists=True,
        argstr='%s',
        mandatory=True,
        position=-2,
        desc='the image containing the source data.'
        'The type of data required depends on the type of tracking as set in the preceeding argument. For DT methods, '
        'the base DWI are needed. For SD methods, the SH harmonic coefficients of the FOD are needed.'
    )

    seed_xor = ['seed_file', 'seed_spec']
    seed_file = File(
        exists=True, argstr='-seed %s', desc='seed file', xor=seed_xor)
    seed_spec = traits.List(
        traits.Float,
        desc='seed specification in mm and radius (x y z r)',
        position=2,
        argstr='-seed %s',
        minlen=4,
        maxlen=4,
        sep=',',
        units='mm',
        xor=seed_xor)

    include_xor = ['include_file', 'include_spec']
    include_file = File(
        exists=True,
        argstr='-include %s',
        desc='inclusion file',
        xor=include_xor)
    include_spec = traits.List(
        traits.Float,
        desc='inclusion specification in mm and radius (x y z r)',
        position=2,
        argstr='-include %s',
        minlen=4,
        maxlen=4,
        sep=',',
        units='mm',
        xor=include_xor)

    exclude_xor = ['exclude_file', 'exclude_spec']
    exclude_file = File(
        exists=True,
        argstr='-exclude %s',
        desc='exclusion file',
        xor=exclude_xor)
    exclude_spec = traits.List(
        traits.Float,
        desc='exclusion specification in mm and radius (x y z r)',
        position=2,
        argstr='-exclude %s',
        minlen=4,
        maxlen=4,
        sep=',',
        units='mm',
        xor=exclude_xor)

    mask_xor = ['mask_file', 'mask_spec']
    mask_file = File(
        exists=True,
        argstr='-mask %s',
        desc='mask file. Only tracks within mask.',
        xor=mask_xor)
    mask_spec = traits.List(
        traits.Float,
        desc=
        'Mask specification in mm and radius (x y z r). Tracks will be terminated when they leave the ROI.',
        position=2,
        argstr='-mask %s',
        minlen=4,
        maxlen=4,
        sep=',',
        units='mm',
        xor=mask_xor)

    inputmodel = traits.Enum(
        'DT_STREAM',
        'SD_PROB',
        'SD_STREAM',
        argstr='%s',
        desc='input model type',
        usedefault=True,
        position=-3)

    stop = traits.Bool(
        argstr='-stop',
        desc="stop track as soon as it enters any of the include regions.")
    do_not_precompute = traits.Bool(
        argstr='-noprecomputed',
        desc=
        "Turns off precomputation of the legendre polynomial values. Warning: this will slow down the algorithm by a factor of approximately 4."
    )
    unidirectional = traits.Bool(
        argstr='-unidirectional',
        desc=
        "Track from the seed point in one direction only (default is to track in both directions)."
    )
    no_mask_interpolation = traits.Bool(
        argstr='-nomaskinterp',
        desc="Turns off trilinear interpolation of mask images.")

    step_size = traits.Float(
        argstr='-step %s',
        units='mm',
        desc="Set the step size of the algorithm in mm (default is 0.2).")
    minimum_radius_of_curvature = traits.Float(
        argstr='-curvature %s',
        units='mm',
        desc=
        "Set the minimum radius of curvature (default is 2 mm for DT_STREAM, 0 for SD_STREAM, 1 mm for SD_PROB and DT_PROB)"
    )
    desired_number_of_tracks = traits.Int(
        argstr='-number %d',
        desc='Sets the desired number of tracks.'
        'The program will continue to generate tracks until this number of tracks have been selected and written to the output file'
        '(default is 100 for *_STREAM methods, 1000 for *_PROB methods).')
    maximum_number_of_tracks = traits.Int(
        argstr='-maxnum %d',
        desc='Sets the maximum number of tracks to generate.'
        "The program will not generate more tracks than this number, even if the desired number of tracks hasn't yet been reached"
        '(default is 100 x number).')

    minimum_tract_length = traits.Float(
        argstr='-minlength %s',
        units='mm',
        desc=
        "Sets the minimum length of any track in millimeters (default is 10 mm)."
    )
    maximum_tract_length = traits.Float(
        argstr='-length %s',
        units='mm',
        desc=
        "Sets the maximum length of any track in millimeters (default is 200 mm)."
    )

    cutoff_value = traits.Float(
        argstr='-cutoff %s',
        units='NA',
        desc=
        "Set the FA or FOD amplitude cutoff for terminating tracks (default is 0.1)."
    )
    initial_cutoff_value = traits.Float(
        argstr='-initcutoff %s',
        units='NA',
        desc=
        "Sets the minimum FA or FOD amplitude for initiating tracks (default is twice the normal cutoff)."
    )

    initial_direction = traits.List(
        traits.Int,
        desc='Specify the initial tracking direction as a vector',
        argstr='-initdirection %s',
        minlen=2,
        maxlen=2,
        units='voxels')
    out_file = File(
        argstr='%s',
        position=-1,
        name_source=['in_file'],
        name_template='%s_tracked.tck',
        output_name='tracked',
        desc='output data file')


class StreamlineTrackOutputSpec(TraitedSpec):
    tracked = File(
        exists=True, desc='output file containing reconstructed tracts')


class StreamlineTrack(CommandLine):
    """
    Performs tractography using one of the following models:
    'dt_prob', 'dt_stream', 'sd_prob', 'sd_stream',
    Where 'dt' stands for diffusion tensor, 'sd' stands for spherical
    deconvolution, and 'prob' stands for probabilistic.

    Example
    -------

    >>> import nipype.interfaces.mrtrix as mrt
    >>> strack = mrt.StreamlineTrack()
    >>> strack.inputs.inputmodel = 'SD_PROB'
    >>> strack.inputs.in_file = 'data.Bfloat'
    >>> strack.inputs.seed_file = 'seed_mask.nii'
    >>> strack.inputs.mask_file = 'mask.nii'
    >>> strack.cmdline
    'streamtrack -mask mask.nii -seed seed_mask.nii SD_PROB data.Bfloat data_tracked.tck'
    >>> strack.run()                                    # doctest: +SKIP
    """
    _cmd = 'streamtrack'
    input_spec = StreamlineTrackInputSpec
    output_spec = StreamlineTrackOutputSpec


class DiffusionTensorStreamlineTrackInputSpec(StreamlineTrackInputSpec):
    gradient_encoding_file = File(
        exists=True,
        argstr='-grad %s',
        mandatory=True,
        position=-2,
        desc=
        'Gradient encoding, supplied as a 4xN text file with each line is in the format [ X Y Z b ], where [ X Y Z ] describe the direction of the applied gradient, and b gives the b-value in units (1000 s/mm^2). See FSL2MRTrix'
    )


class DiffusionTensorStreamlineTrack(StreamlineTrack):
    """
    Specialized interface to StreamlineTrack. This interface is used for
    streamline tracking from diffusion tensor data, and calls the MRtrix
    function 'streamtrack' with the option 'DT_STREAM'

    Example
    -------

    >>> import nipype.interfaces.mrtrix as mrt
    >>> dtstrack = mrt.DiffusionTensorStreamlineTrack()
    >>> dtstrack.inputs.in_file = 'data.Bfloat'
    >>> dtstrack.inputs.seed_file = 'seed_mask.nii'
    >>> dtstrack.run()                                  # doctest: +SKIP
    """

    input_spec = DiffusionTensorStreamlineTrackInputSpec

    def __init__(self, command=None, **inputs):
        inputs["inputmodel"] = "DT_STREAM"
        return super(DiffusionTensorStreamlineTrack, self).__init__(
            command, **inputs)


class ProbabilisticSphericallyDeconvolutedStreamlineTrackInputSpec(
        StreamlineTrackInputSpec):
    maximum_number_of_trials = traits.Int(
        argstr='-trials %s',
        desc=
        "Set the maximum number of sampling trials at each point (only used for probabilistic tracking)."
    )


class ProbabilisticSphericallyDeconvolutedStreamlineTrack(StreamlineTrack):
    """
    Performs probabilistic tracking using spherically deconvolved data

    Specialized interface to StreamlineTrack. This interface is used for
    probabilistic tracking from spherically deconvolved data, and calls
    the MRtrix function 'streamtrack' with the option 'SD_PROB'

    Example
    -------

    >>> import nipype.interfaces.mrtrix as mrt
    >>> sdprobtrack = mrt.ProbabilisticSphericallyDeconvolutedStreamlineTrack()
    >>> sdprobtrack.inputs.in_file = 'data.Bfloat'
    >>> sdprobtrack.inputs.seed_file = 'seed_mask.nii'
    >>> sdprobtrack.run()                                                       # doctest: +SKIP
    """
    input_spec = ProbabilisticSphericallyDeconvolutedStreamlineTrackInputSpec

    def __init__(self, command=None, **inputs):
        inputs["inputmodel"] = "SD_PROB"
        return super(ProbabilisticSphericallyDeconvolutedStreamlineTrack,
                     self).__init__(command, **inputs)


class SphericallyDeconvolutedStreamlineTrack(StreamlineTrack):
    """
    Performs streamline tracking using spherically deconvolved data

    Specialized interface to StreamlineTrack. This interface is used for
    streamline tracking from spherically deconvolved data, and calls
    the MRtrix function 'streamtrack' with the option 'SD_STREAM'

    Example
    -------

    >>> import nipype.interfaces.mrtrix as mrt
    >>> sdtrack = mrt.SphericallyDeconvolutedStreamlineTrack()
    >>> sdtrack.inputs.in_file = 'data.Bfloat'
    >>> sdtrack.inputs.seed_file = 'seed_mask.nii'
    >>> sdtrack.run()                                          # doctest: +SKIP
    """
    input_spec = StreamlineTrackInputSpec

    def __init__(self, command=None, **inputs):
        inputs["inputmodel"] = "SD_STREAM"
        return super(SphericallyDeconvolutedStreamlineTrack, self).__init__(
            command, **inputs)



# -*- coding: utf-8 -*-
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)
from builtins import range, str, bytes

import os

from ...utils import NUMPY_MMAP

from .base import NipyBaseInterface
from ..base import (TraitedSpec, traits, File, OutputMultiPath,
                    BaseInterfaceInputSpec, isdefined)


class FitGLMInputSpec(BaseInterfaceInputSpec):
    session_info = traits.List(
        minlen=1,
        maxlen=1,
        mandatory=True,
        desc=('Session specific information generated by'
              ' ``modelgen.SpecifyModel``, FitGLM does '
              'not  support multiple runs uless they are '
              'concatenated (see SpecifyModel options)'))
    hrf_model = traits.Enum(
        'Canonical',
        'Canonical With Derivative',
        'FIR',
        desc=("that specifies the hemodynamic reponse "
              "function it can be 'Canonical', 'Canonical "
              "With Derivative' or 'FIR'"),
        usedefault=True)
    drift_model = traits.Enum(
        "Cosine",
        "Polynomial",
        "Blank",
        desc=("string that specifies the desired drift "
              "model, to be chosen among 'Polynomial', "
              "'Cosine', 'Blank'"),
        usedefault=True)
    TR = traits.Float(mandatory=True)
    model = traits.Enum(
        "ar1",
        "spherical",
        desc=("autoregressive mode is available only for the "
              "kalman method"),
        usedefault=True)
    method = traits.Enum(
        "kalman",
        "ols",
        desc=("method to fit the model, ols or kalma; kalman "
              "is more time consuming but it supports "
              "autoregressive model"),
        usedefault=True)
    mask = File(
        exists=True,
        desc=("restrict the fitting only to the region defined "
              "by this mask"))
    normalize_design_matrix = traits.Bool(
        False,
        desc=("normalize (zscore) the "
              "regressors before fitting"),
        usedefault=True)
    save_residuals = traits.Bool(False, usedefault=True)
    plot_design_matrix = traits.Bool(False, usedefault=True)


class FitGLMOutputSpec(TraitedSpec):
    beta = File(exists=True)
    nvbeta = traits.Any()
    s2 = File(exists=True)
    dof = traits.Any()
    constants = traits.Any()
    axis = traits.Any()
    reg_names = traits.List()
    residuals = File()
    a = File(exists=True)


class FitGLM(NipyBaseInterface):
    '''
    Fit GLM model based on the specified design. Supports only single or concatenated runs.
    '''
    input_spec = FitGLMInputSpec
    output_spec = FitGLMOutputSpec

    def _run_interface(self, runtime):
        import nibabel as nb
        import numpy as np
        import nipy.modalities.fmri.glm as GLM
        import nipy.modalities.fmri.design_matrix as dm
        try:
            BlockParadigm = dm.BlockParadigm
        except AttributeError:
            from nipy.modalities.fmri.experimental_paradigm import BlockParadigm

        session_info = self.inputs.session_info

        functional_runs = self.inputs.session_info[0]['scans']
        if isinstance(functional_runs, (str, bytes)):
            functional_runs = [functional_runs]
        nii = nb.load(functional_runs[0])
        data = nii.get_data()

        if isdefined(self.inputs.mask):
            mask = nb.load(self.inputs.mask).get_data() > 0
        else:
            mask = np.ones(nii.shape[:3]) == 1

        timeseries = data.copy()[mask, :]
        del data

        for functional_run in functional_runs[1:]:
            nii = nb.load(functional_run, mmap=NUMPY_MMAP)
            data = nii.get_data()
            npdata = data.copy()
            del data
            timeseries = np.concatenate((timeseries, npdata[mask, :]), axis=1)
            del npdata

        nscans = timeseries.shape[1]

        if 'hpf' in list(session_info[0].keys()):
            hpf = session_info[0]['hpf']
            drift_model = self.inputs.drift_model
        else:
            hpf = 0
            drift_model = "Blank"

        reg_names = []
        for reg in session_info[0]['regress']:
            reg_names.append(reg['name'])

        reg_vals = np.zeros((nscans, len(reg_names)))
        for i in range(len(reg_names)):
            reg_vals[:, i] = np.array(
                session_info[0]['regress'][i]['val']).reshape(1, -1)

        frametimes = np.linspace(0, (nscans - 1) * self.inputs.TR, nscans)

        conditions = []
        onsets = []
        duration = []

        for i, cond in enumerate(session_info[0]['cond']):
            onsets += cond['onset']
            conditions += [cond['name']] * len(cond['onset'])
            if len(cond['duration']) == 1:
                duration += cond['duration'] * len(cond['onset'])
            else:
                duration += cond['duration']

        if conditions:
            paradigm = BlockParadigm(
                con_id=conditions, onset=onsets, duration=duration)
        else:
            paradigm = None
        design_matrix, self._reg_names = dm.dmtx_light(
            frametimes,
            paradigm,
            drift_model=drift_model,
            hfcut=hpf,
            hrf_model=self.inputs.hrf_model,
            add_regs=reg_vals,
            add_reg_names=reg_names)
        if self.inputs.normalize_design_matrix:
            for i in range(len(self._reg_names) - 1):
                design_matrix[:, i] = ((
                    design_matrix[:, i] - design_matrix[:, i].mean()) /
                                       design_matrix[:, i].std())

        if self.inputs.plot_design_matrix:
            import pylab
            pylab.pcolor(design_matrix)
            pylab.savefig("design_matrix.pdf")
            pylab.close()
            pylab.clf()

        glm = GLM.GeneralLinearModel()
        glm.fit(
            timeseries.T,
            design_matrix,
            method=self.inputs.method,
            model=self.inputs.model)

        self._beta_file = os.path.abspath("beta.nii")
        beta = np.zeros(mask.shape + (glm.beta.shape[0], ))
        beta[mask, :] = glm.beta.T
        nb.save(nb.Nifti1Image(beta, nii.affine), self._beta_file)

        self._s2_file = os.path.abspath("s2.nii")
        s2 = np.zeros(mask.shape)
        s2[mask] = glm.s2
        nb.save(nb.Nifti1Image(s2, nii.affine), self._s2_file)

        if self.inputs.save_residuals:
            explained = np.dot(design_matrix, glm.beta)
            residuals = np.zeros(mask.shape + (nscans, ))
            residuals[mask, :] = timeseries - explained.T
            self._residuals_file = os.path.abspath("residuals.nii")
            nb.save(
                nb.Nifti1Image(residuals, nii.affine), self._residuals_file)

        self._nvbeta = glm.nvbeta
        self._dof = glm.dof
        self._constants = glm._constants
        self._axis = glm._axis
        if self.inputs.model == "ar1":
            self._a_file = os.path.abspath("a.nii")
            a = np.zeros(mask.shape)
            a[mask] = glm.a.squeeze()
            nb.save(nb.Nifti1Image(a, nii.affine), self._a_file)
        self._model = glm.model
        self._method = glm.method

        return runtime

    def _list_outputs(self):
        outputs = self._outputs().get()
        outputs["beta"] = self._beta_file
        outputs["nvbeta"] = self._nvbeta
        outputs["s2"] = self._s2_file
        outputs["dof"] = self._dof
        outputs["constants"] = self._constants
        outputs["axis"] = self._axis
        outputs["reg_names"] = self._reg_names
        if self.inputs.model == "ar1":
            outputs["a"] = self._a_file
        if self.inputs.save_residuals:
            outputs["residuals"] = self._residuals_file
        return outputs


class EstimateContrastInputSpec(BaseInterfaceInputSpec):
    contrasts = traits.List(
        traits.Either(
            traits.Tuple(traits.Str, traits.Enum('T'), traits.List(traits.Str),
                         traits.List(traits.Float)),
            traits.Tuple(traits.Str, traits.Enum('T'), traits.List(traits.Str),
                         traits.List(traits.Float), traits.List(traits.Float)),
            traits.Tuple(traits.Str, traits.Enum('F'),
                         traits.List(
                             traits.Either(
                                 traits.Tuple(traits.Str, traits.Enum('T'),
                                              traits.List(traits.Str),
                                              traits.List(traits.Float)),
                                 traits.Tuple(traits.Str, traits.Enum('T'),
                                              traits.List(traits.Str),
                                              traits.List(traits.Float),
                                              traits.List(traits.Float)))))),
        desc="""List of contrasts with each contrast being a list of the form:
            [('name', 'stat', [condition list], [weight list], [session list])]. if
            session list is None or not provided, all sessions are used. For F
            contrasts, the condition list should contain previously defined
            T-contrasts.""",
        mandatory=True)
    beta = File(
        exists=True,
        desc="beta coefficients of the fitted model",
        mandatory=True)
    nvbeta = traits.Any(mandatory=True)
    s2 = File(
        exists=True, desc="squared variance of the residuals", mandatory=True)
    dof = traits.Any(desc="degrees of freedom", mandatory=True)
    constants = traits.Any(mandatory=True)
    axis = traits.Any(mandatory=True)
    reg_names = traits.List(mandatory=True)
    mask = File(exists=True)


class EstimateContrastOutputSpec(TraitedSpec):
    stat_maps = OutputMultiPath(File(exists=True))
    z_maps = OutputMultiPath(File(exists=True))
    p_maps = OutputMultiPath(File(exists=True))


class EstimateContrast(NipyBaseInterface):
    '''
    Estimate contrast of a fitted model.
    '''
    input_spec = EstimateContrastInputSpec
    output_spec = EstimateContrastOutputSpec

    def _run_interface(self, runtime):
        import nibabel as nb
        import numpy as np
        import nipy.modalities.fmri.glm as GLM

        beta_nii = nb.load(self.inputs.beta)
        if isdefined(self.inputs.mask):
            mask = nb.load(self.inputs.mask).get_data() > 0
        else:
            mask = np.ones(beta_nii.shape[:3]) == 1

        glm = GLM.GeneralLinearModel()
        nii = nb.load(self.inputs.beta)
        glm.beta = beta_nii.get_data().copy()[mask, :].T
        glm.nvbeta = self.inputs.nvbeta
        glm.s2 = nb.load(self.inputs.s2).get_data().copy()[mask]
        glm.dof = self.inputs.dof
        glm._axis = self.inputs.axis
        glm._constants = self.inputs.constants

        reg_names = self.inputs.reg_names

        self._stat_maps = []
        self._p_maps = []
        self._z_maps = []
        for contrast_def in self.inputs.contrasts:
            name = contrast_def[0]
            contrast = np.zeros(len(reg_names))

            for i, reg_name in enumerate(reg_names):
                if reg_name in contrast_def[2]:
                    idx = contrast_def[2].index(reg_name)
                    contrast[i] = contrast_def[3][idx]

            est_contrast = glm.contrast(contrast)

            stat_map = np.zeros(mask.shape)
            stat_map[mask] = est_contrast.stat().T
            stat_map_file = os.path.abspath(name + "_stat_map.nii")
            nb.save(nb.Nifti1Image(stat_map, nii.affine), stat_map_file)
            self._stat_maps.append(stat_map_file)

            p_map = np.zeros(mask.shape)
            p_map[mask] = est_contrast.pvalue().T
            p_map_file = os.path.abspath(name + "_p_map.nii")
            nb.save(nb.Nifti1Image(p_map, nii.affine), p_map_file)
            self._p_maps.append(p_map_file)

            z_map = np.zeros(mask.shape)
            z_map[mask] = est_contrast.zscore().T
            z_map_file = os.path.abspath(name + "_z_map.nii")
            nb.save(nb.Nifti1Image(z_map, nii.affine), z_map_file)
            self._z_maps.append(z_map_file)

        return runtime

    def _list_outputs(self):
        outputs = self._outputs().get()
        outputs["stat_maps"] = self._stat_maps
        outputs["p_maps"] = self._p_maps
        outputs["z_maps"] = self._z_maps
        return outputs



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..specialized import BRAINSCreateLabelMapFromProbabilityMaps


def test_BRAINSCreateLabelMapFromProbabilityMaps_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        cleanLabelVolume=dict(
            argstr='--cleanLabelVolume %s',
            hash_files=False,
        ),
        dirtyLabelVolume=dict(
            argstr='--dirtyLabelVolume %s',
            hash_files=False,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        foregroundPriors=dict(
            argstr='--foregroundPriors %s',
            sep=',',
        ),
        inclusionThreshold=dict(argstr='--inclusionThreshold %f', ),
        inputProbabilityVolume=dict(argstr='--inputProbabilityVolume %s...', ),
        nonAirRegionMask=dict(
            argstr='--nonAirRegionMask %s',
            extensions=None,
        ),
        priorLabelCodes=dict(
            argstr='--priorLabelCodes %s',
            sep=',',
        ),
    )
    inputs = BRAINSCreateLabelMapFromProbabilityMaps.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_BRAINSCreateLabelMapFromProbabilityMaps_outputs():
    output_map = dict(
        cleanLabelVolume=dict(extensions=None, ),
        dirtyLabelVolume=dict(extensions=None, ),
    )
    outputs = BRAINSCreateLabelMapFromProbabilityMaps.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""The regutils module provides classes for interfacing with the `niftyreg
<http://niftyreg.sourceforge.net>`_ utility command line tools.

The interfaces were written to work with niftyreg version 1.5.10
"""

from __future__ import (print_function, division, unicode_literals,
                        absolute_import)
from builtins import len, open, property, super
import os

from ..base import TraitedSpec, File, traits, isdefined
from .base import get_custom_path, NiftyRegCommand, NiftyRegCommandInputSpec
from ...utils.filemanip import split_filename


class RegResampleInputSpec(NiftyRegCommandInputSpec):
    """ Input Spec for RegResample. """
    # Input reference file
    ref_file = File(
        exists=True,
        desc='The input reference/target image',
        argstr='-ref %s',
        mandatory=True)
    # Input floating file
    flo_file = File(
        exists=True,
        desc='The input floating/source image',
        argstr='-flo %s',
        mandatory=True)
    # Input deformation field
    trans_file = File(
        exists=True, desc='The input transformation file', argstr='-trans %s')

    type = traits.Enum(
        'res',
        'blank',
        argstr='-%s',
        position=-2,
        usedefault=True,
        desc='Type of output')

    # Output file name
    out_file = File(
        name_source=['flo_file'],
        name_template='%s',
        argstr='%s',
        position=-1,
        desc='The output filename of the transformed image')

    # Interpolation type
    inter_val = traits.Enum(
        'NN',
        'LIN',
        'CUB',
        'SINC',
        desc='Interpolation type',
        argstr='-inter %d')

    # Padding value
    pad_val = traits.Float(desc='Padding value', argstr='-pad %f')

    # Tensor flag
    tensor_flag = traits.Bool(desc='Resample Tensor Map', argstr='-tensor ')

    # Verbosity off
    verbosity_off_flag = traits.Bool(
        argstr='-voff', desc='Turn off verbose output')
    # PSF flag
    desc = 'Perform the resampling in two steps to resample an image to a \
lower resolution'

    psf_flag = traits.Bool(argstr='-psf', desc=desc)
    desc = 'Minimise the matrix metric (0) or the determinant (1) when \
estimating the PSF [0]'

    psf_alg = traits.Enum(0, 1, argstr='-psf_alg %d', desc=desc)


class RegResampleOutputSpec(TraitedSpec):
    """ Output Spec for RegResample. """
    out_file = File(desc='The output filename of the transformed image')


class RegResample(NiftyRegCommand):
    """Interface for executable reg_resample from NiftyReg platform.

    Tool to resample floating image in the space of a defined reference image
    given a transformation parametrisation generated by reg_aladin, reg_f3d or
    reg_transform

    `Source code <https://cmiclab.cs.ucl.ac.uk/mmodat/niftyreg>`_

    Examples
    --------
    >>> from nipype.interfaces import niftyreg
    >>> node = niftyreg.RegResample()
    >>> node.inputs.ref_file = 'im1.nii'
    >>> node.inputs.flo_file = 'im2.nii'
    >>> node.inputs.trans_file = 'warpfield.nii'
    >>> node.inputs.inter_val = 'LIN'
    >>> node.inputs.omp_core_val = 4
    >>> node.cmdline
    'reg_resample -flo im2.nii -inter 1 -omp 4 -ref im1.nii -trans \
warpfield.nii -res im2_res.nii.gz'

    """
    _cmd = get_custom_path('reg_resample')
    input_spec = RegResampleInputSpec
    output_spec = RegResampleOutputSpec

    # Need this overload to properly constraint the interpolation type input
    def _format_arg(self, name, spec, value):
        if name == 'inter_val':
            inter_val = {'NN': 0, 'LIN': 1, 'CUB': 3, 'SINC': 4}
            return spec.argstr % inter_val[value]
        else:
            return super(RegResample, self)._format_arg(name, spec, value)

    def _overload_extension(self, value, name=None):
        path, base, _ = split_filename(value)
        suffix = self.inputs.type
        return os.path.join(path, '{0}_{1}.nii.gz'.format(base, suffix))


class RegJacobianInputSpec(NiftyRegCommandInputSpec):
    """ Input Spec for RegJacobian. """
    # Reference file name
    desc = 'Reference/target file (required if specifying CPP transformations.'
    ref_file = File(exists=True, desc=desc, argstr='-ref %s')
    # Input transformation file
    trans_file = File(
        exists=True,
        desc='The input non-rigid transformation',
        argstr='-trans %s',
        mandatory=True)
    type = traits.Enum(
        'jac',
        'jacL',
        'jacM',
        usedefault=True,
        argstr='-%s',
        position=-2,
        desc='Type of jacobian outcome')
    out_file = File(
        name_source=['trans_file'],
        name_template='%s',
        desc='The output jacobian determinant file name',
        argstr='%s',
        position=-1)


class RegJacobianOutputSpec(TraitedSpec):
    """ Output Spec for RegJacobian. """
    out_file = File(desc='The output file')


class RegJacobian(NiftyRegCommand):
    """Interface for executable reg_resample from NiftyReg platform.

    Tool to generate Jacobian determinant maps from transformation
    parametrisation generated by reg_f3d

    `Source code <https://cmiclab.cs.ucl.ac.uk/mmodat/niftyreg>`_

    Examples
    --------
    >>> from nipype.interfaces import niftyreg
    >>> node = niftyreg.RegJacobian()
    >>> node.inputs.ref_file = 'im1.nii'
    >>> node.inputs.trans_file = 'warpfield.nii'
    >>> node.inputs.omp_core_val = 4
    >>> node.cmdline
    'reg_jacobian -omp 4 -ref im1.nii -trans warpfield.nii -jac \
warpfield_jac.nii.gz'

    """
    _cmd = get_custom_path('reg_jacobian')
    input_spec = RegJacobianInputSpec
    output_spec = RegJacobianOutputSpec

    def _overload_extension(self, value, name=None):
        path, base, _ = split_filename(value)
        suffix = self.inputs.type
        return os.path.join(path, '{0}_{1}.nii.gz'.format(base, suffix))


class RegToolsInputSpec(NiftyRegCommandInputSpec):
    """ Input Spec for RegTools. """
    # Input image file
    in_file = File(
        exists=True,
        desc='The input image file path',
        argstr='-in %s',
        mandatory=True)

    # Output file path
    out_file = File(
        name_source=['in_file'],
        name_template='%s_tools.nii.gz',
        desc='The output file name',
        argstr='-out %s')

    # Make the output image isotropic
    iso_flag = traits.Bool(argstr='-iso', desc='Make output image isotropic')

    # Set scale, slope to 0 and 1.
    noscl_flag = traits.Bool(
        argstr='-noscl', desc='Set scale, slope to 0 and 1')

    # Values outside the mask are set to NaN
    mask_file = File(
        exists=True,
        desc='Values outside the mask are set to NaN',
        argstr='-nan %s')

    # Threshold the input image
    desc = 'Binarise the input image with the given threshold'
    thr_val = traits.Float(desc=desc, argstr='-thr %f')

    # Binarise the input image
    bin_flag = traits.Bool(argstr='-bin', desc='Binarise the input image')

    # Compute the mean RMS between the two images
    rms_val = File(
        exists=True,
        desc='Compute the mean RMS between the images',
        argstr='-rms %s')

    # Perform division by image or value
    div_val = traits.Either(
        traits.Float,
        File(exists=True),
        desc='Divide the input by image or value',
        argstr='-div %s')

    # Perform multiplication by image or value
    mul_val = traits.Either(
        traits.Float,
        File(exists=True),
        desc='Multiply the input by image or value',
        argstr='-mul %s')

    # Perform addition by image or value
    add_val = traits.Either(
        traits.Float,
        File(exists=True),
        desc='Add to the input image or value',
        argstr='-add %s')

    # Perform subtraction by image or value
    sub_val = traits.Either(
        traits.Float,
        File(exists=True),
        desc='Add to the input image or value',
        argstr='-sub %s')

    # Downsample the image by a factor of 2.
    down_flag = traits.Bool(
        desc='Downsample the image by a factor of 2', argstr='-down')

    # Smoothing using spline kernel
    desc = 'Smooth the input image using a cubic spline kernel'
    smo_s_val = traits.Tuple(
        traits.Float,
        traits.Float,
        traits.Float,
        desc=desc,
        argstr='-smoS %f %f %f')

    # Change the resolution of the input image
    chg_res_val = traits.Tuple(
        traits.Float,
        traits.Float,
        traits.Float,
        desc='Change the resolution of the input image',
        argstr='-chgres %f %f %f')

    # Smoothing using Gaussian kernel
    desc = 'Smooth the input image using a Gaussian kernel'
    smo_g_val = traits.Tuple(
        traits.Float,
        traits.Float,
        traits.Float,
        desc=desc,
        argstr='-smoG %f %f %f')

    # Interpolation type
    inter_val = traits.Enum(
        'NN',
        'LIN',
        'CUB',
        'SINC',
        desc='Interpolation order to use to warp the floating image',
        argstr='-interp %d')


class RegToolsOutputSpec(TraitedSpec):
    """ Output Spec for RegTools. """
    out_file = File(desc='The output file', exists=True)


class RegTools(NiftyRegCommand):
    """Interface for executable reg_tools from NiftyReg platform.

    Tool delivering various actions related to registration such as
    resampling the input image to a chosen resolution or remove the nan and
    inf in the input image by a specified value.

    `Source code <https://cmiclab.cs.ucl.ac.uk/mmodat/niftyreg>`_

    Examples
    --------
    >>> from nipype.interfaces import niftyreg
    >>> node = niftyreg.RegTools()
    >>> node.inputs.in_file = 'im1.nii'
    >>> node.inputs.mul_val = 4
    >>> node.inputs.omp_core_val = 4
    >>> node.cmdline
    'reg_tools -in im1.nii -mul 4.0 -omp 4 -out im1_tools.nii.gz'

    """
    _cmd = get_custom_path('reg_tools')
    input_spec = RegToolsInputSpec
    output_spec = RegToolsOutputSpec
    _suffix = '_tools'

    # Need this overload to properly constraint the interpolation type input
    def _format_arg(self, name, spec, value):
        if name == 'inter_val':
            inter_val = {'NN': 0, 'LIN': 1, 'CUB': 3, 'SINC': 4}
            return spec.argstr % inter_val[value]
        else:
            return super(RegTools, self)._format_arg(name, spec, value)


class RegAverageInputSpec(NiftyRegCommandInputSpec):
    """ Input Spec for RegAverage. """
    avg_files = traits.List(
        File(exist=True),
        position=1,
        argstr='-avg %s',
        sep=' ',
        xor=[
            'avg_lts_files', 'avg_ref_file', 'demean1_ref_file',
            'demean2_ref_file', 'demean3_ref_file', 'warp_files'
        ],
        desc='Averaging of images/affine transformations')

    desc = 'Robust average of affine transformations'
    avg_lts_files = traits.List(
        File(exist=True),
        position=1,
        argstr='-avg_lts %s',
        sep=' ',
        xor=[
            'avg_files', 'avg_ref_file', 'demean1_ref_file',
            'demean2_ref_file', 'demean3_ref_file', 'warp_files'
        ],
        desc=desc)

    desc = 'All input images are resampled into the space of <reference image>\
 and averaged. A cubic spline interpolation scheme is used for resampling'

    avg_ref_file = File(
        position=1,
        argstr='-avg_tran %s',
        xor=[
            'avg_files', 'avg_lts_files', 'demean1_ref_file',
            'demean2_ref_file', 'demean3_ref_file'
        ],
        requires=['warp_files'],
        desc=desc)

    desc = 'Average images and demean average image that have affine \
transformations to a common space'

    demean1_ref_file = File(
        position=1,
        argstr='-demean1 %s',
        xor=[
            'avg_files', 'avg_lts_files', 'avg_ref_file', 'demean2_ref_file',
            'demean3_ref_file'
        ],
        requires=['warp_files'],
        desc=desc)

    desc = 'Average images and demean average image that have non-rigid \
transformations to a common space'

    demean2_ref_file = File(
        position=1,
        argstr='-demean2 %s',
        xor=[
            'avg_files', 'avg_lts_files', 'avg_ref_file', 'demean1_ref_file',
            'demean3_ref_file'
        ],
        requires=['warp_files'],
        desc=desc)

    desc = 'Average images and demean average image that have linear and \
non-rigid transformations to a common space'

    demean3_ref_file = File(
        position=1,
        argstr='-demean3 %s',
        xor=[
            'avg_files', 'avg_lts_files', 'avg_ref_file', 'demean1_ref_file',
            'demean2_ref_file'
        ],
        requires=['warp_files'],
        desc=desc)

    desc = 'transformation files and floating image pairs/triplets to the \
reference space'

    warp_files = traits.List(
        File(exist=True),
        position=-1,
        argstr='%s',
        sep=' ',
        xor=['avg_files', 'avg_lts_files'],
        desc=desc)

    out_file = File(
        genfile=True, position=0, desc='Output file name', argstr='%s')


class RegAverageOutputSpec(TraitedSpec):
    """ Output Spec for RegAverage. """
    out_file = File(desc='Output file name')


class RegAverage(NiftyRegCommand):
    """Interface for executable reg_average from NiftyReg platform.

    Compute average matrix or image from a list of matrices or image.
    The tool can be use to resample images given input transformation
    parametrisation as well as to demean transformations in Euclidean or
    log-Euclidean space.

    This interface is different than the others in the way that the options
    will be written in a command file that is given as a parameter.

    `Source code <https://cmiclab.cs.ucl.ac.uk/mmodat/niftyreg>`_

    Examples
    --------

    >>> from nipype.interfaces import niftyreg
    >>> node = niftyreg.RegAverage()
    >>> one_file = 'im1.nii'
    >>> two_file = 'im2.nii'
    >>> three_file = 'im3.nii'
    >>> node.inputs.avg_files = [one_file, two_file, three_file]
    >>> node.cmdline  # doctest: +ELLIPSIS
    'reg_average --cmd_file .../reg_average_cmd'
    """
    _cmd = get_custom_path('reg_average')
    input_spec = RegAverageInputSpec
    output_spec = RegAverageOutputSpec
    _suffix = 'avg_out'

    def _gen_filename(self, name):
        if name == 'out_file':
            if isdefined(self.inputs.avg_lts_files):
                return self._gen_fname(self._suffix, ext='.txt')
            elif isdefined(self.inputs.avg_files):
                _, _, _ext = split_filename(self.inputs.avg_files[0])
                if _ext not in ['.nii', '.nii.gz', '.hdr', '.img', '.img.gz']:
                    return self._gen_fname(self._suffix, ext=_ext)
            return self._gen_fname(self._suffix, ext='.nii.gz')

        return None

    def _list_outputs(self):
        outputs = self.output_spec().get()

        if isdefined(self.inputs.out_file):
            outputs['out_file'] = self.inputs.out_file
        else:
            outputs['out_file'] = self._gen_filename('out_file')

        return outputs

    @property
    def cmdline(self):
        """ Rewrite the cmdline to write options in text_file."""
        argv = super(RegAverage, self).cmdline
        reg_average_cmd = os.path.join(os.getcwd(), 'reg_average_cmd')
        with open(reg_average_cmd, 'w') as f:
            f.write(argv)
        return '%s --cmd_file %s' % (self.cmd, reg_average_cmd)


class RegTransformInputSpec(NiftyRegCommandInputSpec):
    """ Input Spec for RegTransform. """
    ref1_file = File(
        exists=True,
        desc='The input reference/target image',
        argstr='-ref %s',
        position=0)

    ref2_file = File(
        exists=True,
        desc='The input second reference/target image',
        argstr='-ref2 %s',
        position=1,
        requires=['ref1_file'])

    def_input = File(
        exists=True,
        argstr='-def %s',
        position=-2,
        desc='Compute deformation field from transformation',
        xor=[
            'disp_input', 'flow_input', 'comp_input', 'upd_s_form_input',
            'inv_aff_input', 'inv_nrr_input', 'half_input', 'make_aff_input',
            'aff_2_rig_input', 'flirt_2_nr_input'
        ])

    disp_input = File(
        exists=True,
        argstr='-disp %s',
        position=-2,
        desc='Compute displacement field from transformation',
        xor=[
            'def_input', 'flow_input', 'comp_input', 'upd_s_form_input',
            'inv_aff_input', 'inv_nrr_input', 'half_input', 'make_aff_input',
            'aff_2_rig_input', 'flirt_2_nr_input'
        ])

    flow_input = File(
        exists=True,
        argstr='-flow %s',
        position=-2,
        desc='Compute flow field from spline SVF',
        xor=[
            'def_input', 'disp_input', 'comp_input', 'upd_s_form_input',
            'inv_aff_input', 'inv_nrr_input', 'half_input', 'make_aff_input',
            'aff_2_rig_input', 'flirt_2_nr_input'
        ])

    comp_input = File(
        exists=True,
        argstr='-comp %s',
        position=-3,
        desc='compose two transformations',
        xor=[
            'def_input', 'disp_input', 'flow_input', 'upd_s_form_input',
            'inv_aff_input', 'inv_nrr_input', 'half_input', 'make_aff_input',
            'aff_2_rig_input', 'flirt_2_nr_input'
        ],
        requires=['comp_input2'])

    comp_input2 = File(
        exists=True,
        argstr='%s',
        position=-2,
        desc='compose two transformations')

    desc = 'Update s-form using the affine transformation'
    upd_s_form_input = File(
        exists=True,
        argstr='-updSform %s',
        position=-3,
        desc=desc,
        xor=[
            'def_input', 'disp_input', 'flow_input', 'comp_input',
            'inv_aff_input', 'inv_nrr_input', 'half_input', 'make_aff_input',
            'aff_2_rig_input', 'flirt_2_nr_input'
        ],
        requires=['upd_s_form_input2'])

    desc = 'Update s-form using the affine transformation'
    upd_s_form_input2 = File(
        exists=True,
        argstr='%s',
        position=-2,
        desc=desc,
        requires=['upd_s_form_input'])

    inv_aff_input = File(
        exists=True,
        argstr='-invAff %s',
        position=-2,
        desc='Invert an affine transformation',
        xor=[
            'def_input', 'disp_input', 'flow_input', 'comp_input',
            'upd_s_form_input', 'inv_nrr_input', 'half_input',
            'make_aff_input', 'aff_2_rig_input', 'flirt_2_nr_input'
        ])

    inv_nrr_input = traits.Tuple(
        File(exists=True),
        File(exists=True),
        desc='Invert a non-linear transformation',
        argstr='-invNrr %s %s',
        position=-2,
        xor=[
            'def_input', 'disp_input', 'flow_input', 'comp_input',
            'upd_s_form_input', 'inv_aff_input', 'half_input',
            'make_aff_input', 'aff_2_rig_input', 'flirt_2_nr_input'
        ])

    half_input = File(
        exists=True,
        argstr='-half %s',
        position=-2,
        desc='Half way to the input transformation',
        xor=[
            'def_input', 'disp_input', 'flow_input', 'comp_input',
            'upd_s_form_input', 'inv_aff_input', 'inv_nrr_input',
            'make_aff_input', 'aff_2_rig_input', 'flirt_2_nr_input'
        ])

    argstr_tmp = '-makeAff %f %f %f %f %f %f %f %f %f %f %f %f'
    make_aff_input = traits.Tuple(
        traits.Float,
        traits.Float,
        traits.Float,
        traits.Float,
        traits.Float,
        traits.Float,
        traits.Float,
        traits.Float,
        traits.Float,
        traits.Float,
        traits.Float,
        traits.Float,
        argstr=argstr_tmp,
        position=-2,
        desc='Make an affine transformation matrix',
        xor=[
            'def_input', 'disp_input', 'flow_input', 'comp_input',
            'upd_s_form_input', 'inv_aff_input', 'inv_nrr_input', 'half_input',
            'aff_2_rig_input', 'flirt_2_nr_input'
        ])

    desc = 'Extract the rigid component from affine transformation'
    aff_2_rig_input = File(
        exists=True,
        argstr='-aff2rig %s',
        position=-2,
        desc=desc,
        xor=[
            'def_input', 'disp_input', 'flow_input', 'comp_input',
            'upd_s_form_input', 'inv_aff_input', 'inv_nrr_input', 'half_input',
            'make_aff_input', 'flirt_2_nr_input'
        ])

    desc = 'Convert a FLIRT affine transformation to niftyreg affine \
transformation'

    flirt_2_nr_input = traits.Tuple(
        File(exists=True),
        File(exists=True),
        File(exists=True),
        argstr='-flirtAff2NR %s %s %s',
        position=-2,
        desc=desc,
        xor=[
            'def_input', 'disp_input', 'flow_input', 'comp_input',
            'upd_s_form_input', 'inv_aff_input', 'inv_nrr_input', 'half_input',
            'make_aff_input', 'aff_2_rig_input'
        ])

    out_file = File(
        genfile=True,
        position=-1,
        argstr='%s',
        desc='transformation file to write')


class RegTransformOutputSpec(TraitedSpec):
    """ Output Spec for RegTransform. """
    out_file = File(desc='Output File (transformation in any format)')


class RegTransform(NiftyRegCommand):
    """Interface for executable reg_transform from NiftyReg platform.

    Tools to convert transformation parametrisation from one type to another
    as well as to compose, inverse or half transformations.

    `Source code <https://cmiclab.cs.ucl.ac.uk/mmodat/niftyreg>`_

    Examples
    --------
    >>> from nipype.interfaces import niftyreg
    >>> node = niftyreg.RegTransform()
    >>> node.inputs.def_input = 'warpfield.nii'
    >>> node.inputs.omp_core_val = 4
    >>> node.cmdline  # doctest: +ELLIPSIS
    'reg_transform -omp 4 -def warpfield.nii .../warpfield_trans.nii.gz'

    """
    _cmd = get_custom_path('reg_transform')
    input_spec = RegTransformInputSpec
    output_spec = RegTransformOutputSpec
    _suffix = '_trans'

    def _find_input(self):
        inputs = [
            self.inputs.def_input, self.inputs.disp_input,
            self.inputs.flow_input, self.inputs.comp_input,
            self.inputs.comp_input2, self.inputs.upd_s_form_input,
            self.inputs.inv_aff_input, self.inputs.inv_nrr_input,
            self.inputs.half_input, self.inputs.make_aff_input,
            self.inputs.aff_2_rig_input, self.inputs.flirt_2_nr_input
        ]
        entries = []
        for entry in inputs:
            if isdefined(entry):
                entries.append(entry)
                _, _, ext = split_filename(entry)
                if ext == '.nii' or ext == '.nii.gz' or ext == '.hdr':
                    return entry
        if len(entries):
            return entries[0]
        return None

    def _gen_filename(self, name):
        if name == 'out_file':
            if isdefined(self.inputs.make_aff_input):
                return self._gen_fname(
                    'matrix', suffix=self._suffix, ext='.txt')

            if isdefined(self.inputs.comp_input) and \
               isdefined(self.inputs.comp_input2):
                _, bn1, ext1 = split_filename(self.inputs.comp_input)
                _, _, ext2 = split_filename(self.inputs.comp_input2)
                if ext1 in ['.nii', '.nii.gz', '.hdr', '.img', '.img.gz'] or \
                   ext2 in ['.nii', '.nii.gz', '.hdr', '.img', '.img.gz']:
                    return self._gen_fname(
                        bn1, suffix=self._suffix, ext='.nii.gz')
                else:
                    return self._gen_fname(bn1, suffix=self._suffix, ext=ext1)

            if isdefined(self.inputs.flirt_2_nr_input):
                return self._gen_fname(
                    self.inputs.flirt_2_nr_input[0],
                    suffix=self._suffix,
                    ext='.txt')

            input_to_use = self._find_input()
            _, _, ext = split_filename(input_to_use)
            if ext not in ['.nii', '.nii.gz', '.hdr', '.img', '.img.gz']:
                return self._gen_fname(
                    input_to_use, suffix=self._suffix, ext=ext)
            else:
                return self._gen_fname(
                    input_to_use, suffix=self._suffix, ext='.nii.gz')

        return None

    def _list_outputs(self):
        outputs = self.output_spec().get()

        if isdefined(self.inputs.out_file):
            outputs['out_file'] = self.inputs.out_file
        else:
            outputs['out_file'] = self._gen_filename('out_file')

        return outputs


class RegMeasureInputSpec(NiftyRegCommandInputSpec):
    """ Input Spec for RegMeasure. """
    # Input reference file
    ref_file = File(
        exists=True,
        desc='The input reference/target image',
        argstr='-ref %s',
        mandatory=True)
    # Input floating file
    flo_file = File(
        exists=True,
        desc='The input floating/source image',
        argstr='-flo %s',
        mandatory=True)
    measure_type = traits.Enum(
        'ncc',
        'lncc',
        'nmi',
        'ssd',
        mandatory=True,
        argstr='-%s',
        desc='Measure of similarity to compute')
    out_file = File(
        name_source=['flo_file'],
        name_template='%s',
        argstr='-out %s',
        desc='The output text file containing the measure')


class RegMeasureOutputSpec(TraitedSpec):
    """ Output Spec for RegMeasure. """
    out_file = File(desc='The output text file containing the measure')


class RegMeasure(NiftyRegCommand):
    """Interface for executable reg_measure from NiftyReg platform.

    Given two input images, compute the specified measure(s) of similarity

    `Source code <https://cmiclab.cs.ucl.ac.uk/mmodat/niftyreg>`_

    Examples
    --------
    >>> from nipype.interfaces import niftyreg
    >>> node = niftyreg.RegMeasure()
    >>> node.inputs.ref_file = 'im1.nii'
    >>> node.inputs.flo_file = 'im2.nii'
    >>> node.inputs.measure_type = 'lncc'
    >>> node.inputs.omp_core_val = 4
    >>> node.cmdline
    'reg_measure -flo im2.nii -lncc -omp 4 -out im2_lncc.txt -ref im1.nii'

    """
    _cmd = get_custom_path('reg_measure')
    input_spec = RegMeasureInputSpec
    output_spec = RegMeasureOutputSpec

    def _overload_extension(self, value, name=None):
        path, base, _ = split_filename(value)
        suffix = self.inputs.measure_type
        return os.path.join(path, '{0}_{1}.txt'.format(base, suffix))



# -*- coding: utf-8 -*-
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)

import os
from ...testing import example_data

minc2Dfile = example_data('minc_test_2D_00.mnc')
minc3Dfile = example_data('minc_test_3D_00.mnc')

nlp_config = example_data('minc_nlp.conf')


def nonempty_minc_data(i, shape='2D'):
    return example_data('minc_test_%s_%.2d.mnc' % (
        shape,
        i,
    ))



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..metric import MetricResample


def test_MetricResample_inputs():
    input_map = dict(
        area_metrics=dict(
            argstr='-area-metrics',
            position=5,
            xor=['area_surfs'],
        ),
        area_surfs=dict(
            argstr='-area-surfs',
            position=5,
            xor=['area_metrics'],
        ),
        args=dict(argstr='%s', ),
        current_area=dict(
            argstr='%s',
            extensions=None,
            position=6,
        ),
        current_sphere=dict(
            argstr='%s',
            extensions=None,
            mandatory=True,
            position=1,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr='%s',
            extensions=None,
            mandatory=True,
            position=0,
        ),
        largest=dict(
            argstr='-largest',
            position=10,
        ),
        method=dict(
            argstr='%s',
            mandatory=True,
            position=3,
        ),
        new_area=dict(
            argstr='%s',
            extensions=None,
            position=7,
        ),
        new_sphere=dict(
            argstr='%s',
            extensions=None,
            mandatory=True,
            position=2,
        ),
        out_file=dict(
            argstr='%s',
            extensions=None,
            keep_extension=True,
            name_source=['new_sphere'],
            name_template='%s.out',
            position=4,
        ),
        roi_metric=dict(
            argstr='-current-roi %s',
            extensions=None,
            position=8,
        ),
        valid_roi_out=dict(
            argstr='-valid-roi-out',
            position=9,
        ),
    )
    inputs = MetricResample.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_MetricResample_outputs():
    output_map = dict(
        out_file=dict(extensions=None, ),
        roi_file=dict(extensions=None, ),
    )
    outputs = MetricResample.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..preprocess import ConcatenateLTA


def test_ConcatenateLTA_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_lta1=dict(
            argstr='%s',
            extensions=None,
            mandatory=True,
            position=-3,
        ),
        in_lta2=dict(
            argstr='%s',
            mandatory=True,
            position=-2,
        ),
        invert_1=dict(argstr='-invert1', ),
        invert_2=dict(argstr='-invert2', ),
        invert_out=dict(argstr='-invertout', ),
        out_file=dict(
            argstr='%s',
            extensions=None,
            hash_files=False,
            keep_extension=True,
            name_source=['in_lta1'],
            name_template='%s_concat',
            position=-1,
        ),
        out_type=dict(argstr='-out_type %d', ),
        subject=dict(argstr='-subject %s', ),
        subjects_dir=dict(),
        tal_source_file=dict(
            argstr='-tal %s',
            extensions=None,
            position=-5,
            requires=['tal_template_file'],
        ),
        tal_template_file=dict(
            argstr='%s',
            extensions=None,
            position=-4,
            requires=['tal_source_file'],
        ),
    )
    inputs = ConcatenateLTA.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_ConcatenateLTA_outputs():
    output_map = dict(out_file=dict(extensions=None, ), )
    outputs = ConcatenateLTA.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft = python sts = 4 ts = 4 sw = 4 et:
"""AFNI's svm interfaces."""
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)

from ..base import TraitedSpec, traits, File
from .base import AFNICommand, AFNICommandInputSpec, AFNICommandOutputSpec


class SVMTrainInputSpec(AFNICommandInputSpec):
    # training options
    ttype = traits.Str(
        desc='tname: classification or regression',
        argstr='-type %s',
        mandatory=True)
    in_file = File(
        desc='A 3D+t AFNI brik dataset to be used for training.',
        argstr='-trainvol %s',
        mandatory=True,
        exists=True,
        copyfile=False)
    out_file = File(
        name_template="%s_vectors",
        desc='output sum of weighted linear support vectors file name',
        argstr='-bucket %s',
        suffix='_bucket',
        name_source="in_file")
    model = File(
        name_template="%s_model",
        desc='basename for the brik containing the SVM model',
        argstr='-model %s',
        suffix='_model',
        name_source="in_file")
    alphas = File(
        name_template="%s_alphas",
        desc='output alphas file name',
        argstr='-alpha %s',
        suffix='_alphas',
        name_source="in_file")
    mask = File(
        desc='byte-format brik file used to mask voxels in the analysis',
        argstr='-mask %s',
        position=-1,
        exists=True,
        copyfile=False)
    nomodelmask = traits.Bool(
        desc='Flag to enable the omission of a mask file',
        argstr='-nomodelmask')
    trainlabels = File(
        desc=
        '.1D labels corresponding to the stimulus paradigm for the training data.',
        argstr='-trainlabels %s',
        exists=True)
    censor = File(
        desc=
        '.1D censor file that allows the user to ignore certain samples in the training data.',
        argstr='-censor %s',
        exists=True)
    kernel = traits.Str(
        desc=
        'string specifying type of kernel function:linear, polynomial, rbf, sigmoid',
        argstr='-kernel %s')
    max_iterations = traits.Int(
        desc='Specify the maximum number of iterations for the optimization.',
        argstr='-max_iterations %d')
    w_out = traits.Bool(
        desc='output sum of weighted linear support vectors', argstr='-wout')
    options = traits.Str(desc='additional options for SVM-light', argstr='%s')


class SVMTrainOutputSpec(TraitedSpec):
    out_file = File(desc='sum of weighted linear support vectors file name')
    model = File(desc='brik containing the SVM model file name')
    alphas = File(desc='output alphas file name')


class SVMTrain(AFNICommand):
    """Temporally predictive modeling with the support vector machine
    SVM Train Only
    For complete details, see the `3dsvm Documentation.
    <https://afni.nimh.nih.gov/pub/dist/doc/program_help/3dsvm.html>`_

    Examples
    ========

    >>> from nipype.interfaces import afni as afni
    >>> svmTrain = afni.SVMTrain()
    >>> svmTrain.inputs.in_file = 'run1+orig'
    >>> svmTrain.inputs.trainlabels = 'run1_categories.1D'
    >>> svmTrain.inputs.ttype = 'regression'
    >>> svmTrain.inputs.mask = 'mask.nii'
    >>> svmTrain.inputs.model = 'model_run1'
    >>> svmTrain.inputs.alphas = 'alphas_run1'
    >>> res = svmTrain.run() # doctest: +SKIP

    """

    _cmd = '3dsvm'
    input_spec = SVMTrainInputSpec
    output_spec = SVMTrainOutputSpec
    _additional_metadata = ['suffix']

    def _format_arg(self, name, trait_spec, value):
        return super(SVMTrain, self)._format_arg(name, trait_spec, value)


class SVMTestInputSpec(AFNICommandInputSpec):
    # testing options
    model = traits.Str(
        desc='modname is the basename for the brik containing the SVM model',
        argstr='-model %s',
        mandatory=True)
    in_file = File(
        desc='A 3D or 3D+t AFNI brik dataset to be used for testing.',
        argstr='-testvol %s',
        exists=True,
        mandatory=True)
    out_file = File(
        name_template="%s_predictions",
        desc='filename for .1D prediction file(s).',
        argstr='-predictions %s')
    testlabels = File(
        desc=
        '*true* class category .1D labels for the test dataset. It is used to calculate the prediction accuracy performance',
        exists=True,
        argstr='-testlabels %s')
    classout = traits.Bool(
        desc=
        'Flag to specify that pname files should be integer-valued, corresponding to class category decisions.',
        argstr='-classout')
    nopredcensord = traits.Bool(
        desc=
        'Flag to prevent writing predicted values for censored time-points',
        argstr='-nopredcensord')
    nodetrend = traits.Bool(
        desc=
        'Flag to specify that pname files should not be linearly detrended',
        argstr='-nodetrend')
    multiclass = traits.Bool(
        desc='Specifies multiclass algorithm for classification',
        argstr='-multiclass %s')
    options = traits.Str(desc='additional options for SVM-light', argstr='%s')


class SVMTest(AFNICommand):
    """Temporally predictive modeling with the support vector machine
    SVM Test Only
    For complete details, see the `3dsvm Documentation.
    <https://afni.nimh.nih.gov/pub/dist/doc/program_help/3dsvm.html>`_

    Examples
    ========

    >>> from nipype.interfaces import afni as afni
    >>> svmTest = afni.SVMTest()
    >>> svmTest.inputs.in_file= 'run2+orig'
    >>> svmTest.inputs.model= 'run1+orig_model'
    >>> svmTest.inputs.testlabels= 'run2_categories.1D'
    >>> svmTest.inputs.out_file= 'pred2_model1'
    >>> res = svmTest.run() # doctest: +SKIP

    """
    _cmd = '3dsvm'
    input_spec = SVMTestInputSpec
    output_spec = AFNICommandOutputSpec



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..minc import Dump


def test_Dump_inputs():
    input_map = dict(
        annotations_brief=dict(
            argstr='-b %s',
            xor=('annotations_brief', 'annotations_full'),
        ),
        annotations_full=dict(
            argstr='-f %s',
            xor=('annotations_brief', 'annotations_full'),
        ),
        args=dict(argstr='%s', ),
        coordinate_data=dict(
            argstr='-c',
            xor=('coordinate_data', 'header_data'),
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        header_data=dict(
            argstr='-h',
            xor=('coordinate_data', 'header_data'),
        ),
        input_file=dict(
            argstr='%s',
            extensions=None,
            mandatory=True,
            position=-2,
        ),
        line_length=dict(argstr='-l %d', ),
        netcdf_name=dict(argstr='-n %s', ),
        out_file=dict(
            argstr='> %s',
            extensions=None,
            genfile=True,
            position=-1,
        ),
        output_file=dict(
            extensions=None,
            hash_files=False,
            keep_extension=False,
            name_source=['input_file'],
            name_template='%s_dump.txt',
            position=-1,
        ),
        precision=dict(argstr='%s', ),
        variables=dict(
            argstr='-v %s',
            sep=',',
        ),
    )
    inputs = Dump.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Dump_outputs():
    output_map = dict(output_file=dict(extensions=None, ), )
    outputs = Dump.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..registration import RegistrationSynQuick


def test_RegistrationSynQuick_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        dimension=dict(
            argstr='-d %d',
            usedefault=True,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fixed_image=dict(
            argstr='-f %s...',
            mandatory=True,
        ),
        histogram_bins=dict(
            argstr='-r %d',
            usedefault=True,
        ),
        moving_image=dict(
            argstr='-m %s...',
            mandatory=True,
        ),
        num_threads=dict(
            argstr='-n %d',
            usedefault=True,
        ),
        output_prefix=dict(
            argstr='-o %s',
            usedefault=True,
        ),
        precision_type=dict(
            argstr='-p %s',
            usedefault=True,
        ),
        spline_distance=dict(
            argstr='-s %d',
            usedefault=True,
        ),
        transform_type=dict(
            argstr='-t %s',
            usedefault=True,
        ),
        use_histogram_matching=dict(argstr='-j %d', ),
    )
    inputs = RegistrationSynQuick.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_RegistrationSynQuick_outputs():
    output_map = dict(
        forward_warp_field=dict(extensions=None, ),
        inverse_warp_field=dict(extensions=None, ),
        inverse_warped_image=dict(extensions=None, ),
        out_matrix=dict(extensions=None, ),
        warped_image=dict(extensions=None, ),
    )
    outputs = RegistrationSynQuick.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..preprocess import TCorrMap


def test_TCorrMap_inputs():
    input_map = dict(
        absolute_threshold=dict(
            argstr='-Thresh %f %s',
            extensions=None,
            name_source='in_file',
            suffix='_thresh',
            xor=('absolute_threshold', 'var_absolute_threshold',
                 'var_absolute_threshold_normalize'),
        ),
        args=dict(argstr='%s', ),
        automask=dict(argstr='-automask', ),
        average_expr=dict(
            argstr='-Aexpr %s %s',
            extensions=None,
            name_source='in_file',
            suffix='_aexpr',
            xor=('average_expr', 'average_expr_nonzero', 'sum_expr'),
        ),
        average_expr_nonzero=dict(
            argstr='-Cexpr %s %s',
            extensions=None,
            name_source='in_file',
            suffix='_cexpr',
            xor=('average_expr', 'average_expr_nonzero', 'sum_expr'),
        ),
        bandpass=dict(argstr='-bpass %f %f', ),
        blur_fwhm=dict(argstr='-Gblur %f', ),
        correlation_maps=dict(
            argstr='-CorrMap %s',
            extensions=None,
            name_source='in_file',
        ),
        correlation_maps_masked=dict(
            argstr='-CorrMask %s',
            extensions=None,
            name_source='in_file',
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        expr=dict(),
        histogram=dict(
            argstr='-Hist %d %s',
            extensions=None,
            name_source='in_file',
            suffix='_hist',
        ),
        histogram_bin_numbers=dict(),
        in_file=dict(
            argstr='-input %s',
            copyfile=False,
            extensions=None,
            mandatory=True,
        ),
        mask=dict(
            argstr='-mask %s',
            extensions=None,
        ),
        mean_file=dict(
            argstr='-Mean %s',
            extensions=None,
            name_source='in_file',
            suffix='_mean',
        ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        out_file=dict(
            argstr='-prefix %s',
            extensions=None,
            name_source=['in_file'],
            name_template='%s_afni',
        ),
        outputtype=dict(),
        pmean=dict(
            argstr='-Pmean %s',
            extensions=None,
            name_source='in_file',
            suffix='_pmean',
        ),
        polort=dict(argstr='-polort %d', ),
        qmean=dict(
            argstr='-Qmean %s',
            extensions=None,
            name_source='in_file',
            suffix='_qmean',
        ),
        regress_out_timeseries=dict(
            argstr='-ort %s',
            extensions=None,
        ),
        seeds=dict(
            argstr='-seed %s',
            extensions=None,
            xor='seeds_width',
        ),
        seeds_width=dict(
            argstr='-Mseed %f',
            xor='seeds',
        ),
        sum_expr=dict(
            argstr='-Sexpr %s %s',
            extensions=None,
            name_source='in_file',
            suffix='_sexpr',
            xor=('average_expr', 'average_expr_nonzero', 'sum_expr'),
        ),
        thresholds=dict(),
        var_absolute_threshold=dict(
            argstr='-VarThresh %f %f %f %s',
            extensions=None,
            name_source='in_file',
            suffix='_varthresh',
            xor=('absolute_threshold', 'var_absolute_threshold',
                 'var_absolute_threshold_normalize'),
        ),
        var_absolute_threshold_normalize=dict(
            argstr='-VarThreshN %f %f %f %s',
            extensions=None,
            name_source='in_file',
            suffix='_varthreshn',
            xor=('absolute_threshold', 'var_absolute_threshold',
                 'var_absolute_threshold_normalize'),
        ),
        zmean=dict(
            argstr='-Zmean %s',
            extensions=None,
            name_source='in_file',
            suffix='_zmean',
        ),
    )
    inputs = TCorrMap.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_TCorrMap_outputs():
    output_map = dict(
        absolute_threshold=dict(extensions=None, ),
        average_expr=dict(extensions=None, ),
        average_expr_nonzero=dict(extensions=None, ),
        correlation_maps=dict(extensions=None, ),
        correlation_maps_masked=dict(extensions=None, ),
        histogram=dict(extensions=None, ),
        mean_file=dict(extensions=None, ),
        pmean=dict(extensions=None, ),
        qmean=dict(extensions=None, ),
        sum_expr=dict(extensions=None, ),
        var_absolute_threshold=dict(extensions=None, ),
        var_absolute_threshold_normalize=dict(extensions=None, ),
        zmean=dict(extensions=None, ),
    )
    outputs = TCorrMap.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# -*- coding: utf8 -*-
"""Autogenerated file - DO NOT EDIT
If you spot a bug, please report it on the mailing list and/or change the generator."""

from nipype.interfaces.base import CommandLine, CommandLineInputSpec, SEMLikeCommandLine, TraitedSpec, File, Directory, traits, isdefined, InputMultiPath, OutputMultiPath
import os


class GenerateAverageLmkFileInputSpec(CommandLineInputSpec):
    inputLandmarkFiles = InputMultiPath(
        traits.Str,
        desc="Input landmark files names (.fcsv or .wts)",
        sep=",",
        argstr="--inputLandmarkFiles %s")
    outputLandmarkFile = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc=
        "Ouput landmark file name that includes average values for landmarks (.fcsv or .wts)",
        argstr="--outputLandmarkFile %s")


class GenerateAverageLmkFileOutputSpec(TraitedSpec):
    outputLandmarkFile = File(
        desc=
        "Ouput landmark file name that includes average values for landmarks (.fcsv or .wts)",
        exists=True)


class GenerateAverageLmkFile(SEMLikeCommandLine):
    """title: Average Fiducials

category: Testing

description: This program gets several fcsv file each one contains several landmarks with the same name but slightly different coordinates. For EACH landmark we compute the average coordination.

contributor: Ali Ghayoor

"""

    input_spec = GenerateAverageLmkFileInputSpec
    output_spec = GenerateAverageLmkFileOutputSpec
    _cmd = " GenerateAverageLmkFile "
    _outputs_filenames = {'outputLandmarkFile': 'outputLandmarkFile'}
    _redirect_x = False



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..legacy import buildtemplateparallel


def test_buildtemplateparallel_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        bias_field_correction=dict(argstr='-n 1', ),
        dimension=dict(
            argstr='-d %d',
            position=1,
            usedefault=True,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        gradient_step_size=dict(argstr='-g %f', ),
        in_files=dict(
            argstr='%s',
            mandatory=True,
            position=-1,
        ),
        iteration_limit=dict(
            argstr='-i %d',
            usedefault=True,
        ),
        max_iterations=dict(
            argstr='-m %s',
            sep='x',
        ),
        num_cores=dict(
            argstr='-j %d',
            requires=['parallelization'],
        ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        out_prefix=dict(
            argstr='-o %s',
            usedefault=True,
        ),
        parallelization=dict(
            argstr='-c %d',
            usedefault=True,
        ),
        rigid_body_registration=dict(argstr='-r 1', ),
        similarity_metric=dict(argstr='-s %s', ),
        transformation_model=dict(
            argstr='-t %s',
            usedefault=True,
        ),
        use_first_as_target=dict(),
    )
    inputs = buildtemplateparallel.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_buildtemplateparallel_outputs():
    output_map = dict(
        final_template_file=dict(extensions=None, ),
        subject_outfiles=dict(),
        template_files=dict(),
    )
    outputs = buildtemplateparallel.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value
